/*
 * dmx512_tester.c
 *
 * Created: 12.08.2016 21:35:12
 * Author : karpe
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "pwm.h"
#include "dip.h"


#define USART_BAUDRATE 250000
#define UBRR_VALUE (F_CPU/16/USART_BAUDRATE-1)



#define COUNT_CHANNELS 6

unsigned char data[COUNT_CHANNELS];
unsigned int DMX_Current_Channel;
unsigned int DMX_Start_Channel;


#define LED_PORT PORTB
#define LED_DDR DDRB
#define LED_PIN PB0


void USART_init()
{
	// Set baud rate
	UBRR0H = (uint8_t)(UBRR_VALUE>>8);
	UBRR0L = (uint8_t)UBRR_VALUE;
	/* Set frame format: 8data, 2stop bit */
	//UCSR0C = (1<<URSEL)|(1<<USBS)|(3<<UCSZ00);
	UCSR0C = (1<<USBS0)|(3<<UCSZ00);
	//enable reception
	UCSR0B |= (1<<RXEN0);
	//     
	UCSR0B |= (1<<RXCIE0);
}



ISR(USART_RX_vect)
{
	unsigned char UART_Status = UCSR0A;
	unsigned char UART_Data = UDR0;
	
	//   (Frame error)
	if (UART_Status & (1<<FE0)){
		DMX_Current_Channel = 0;		
		LED_PORT |= (1 << LED_PIN);		
		return;
	}
	
	int index = DMX_Current_Channel - DMX_Start_Channel - 1;
	if ((index >= 0) && (index < COUNT_CHANNELS))
	{
		data[index] = UART_Data;
	}
	DMX_Current_Channel++;
}

ISR(TIMER1_OVF_vect)
{
	static unsigned char count = 0;
	if (count >= 200)
	{
		count = 0;
		LED_PORT &= ~(1 << LED_PIN);		
	}
	count++;
}


/*Disable the watchdog to prevent infinite reset loop */
void WDT_init(void) __attribute__((naked)) __attribute__((section(".init3")));

void WDT_init(void)
{
	MCUSR = 0;
	wdt_disable();
	return;
}


int main(void)
{
	DIP_init();
	
	// Init LED
	LED_DDR |= (1 << LED_PIN);
	
	PWM_init();
	USART_init();
	for (int i = 0; i < COUNT_CHANNELS; i++) data[i] = 0;
	
	// Timer interrupt enable
	TIMSK1 |= (1<<TOIE1);
	
	wdt_enable(WDTO_2S);
	
	sei();
	
    while (1) 
    {
		asm volatile("wdr");
		
		DMX_Start_Channel = DIP_get();
		if (DMX_Start_Channel > (511 - COUNT_CHANNELS))
		{
			DMX_Start_Channel = 511 - COUNT_CHANNELS;
		}
		
		for (int i = 0;  i < COUNT_CHANNELS; i++)
		{
			PWM_set(i, data[i]);			
		}
    }
}

