/*
 * pwm.c
 *
 * Created: 13.08.2016 12:26:42
 *  Author: karpe
 */ 
#include "pwm.h"
#include <avr/io.h>



void PWM_init()
{
	// 6 PWM   
	DDRB |= 1<<PB3; // OC2A
	DDRB |= 1<<PB2; // OC1B
	DDRB |= 1<<PB1; // OC1A
	DDRD |= 1<<PD3; // OC2B
	DDRD |= 1<<PD5; // OC0B
	DDRD |= 1<<PD6; // OC0A
	
	/* Ok, Google. start*/
	TCCR1B |= (1 << WGM12);
	// set Fast PWM mode using ICR1 as TOP

	TCCR1B |= (1 << CS10);
	// START the timer with no prescaler

	// Set mode of operation to FastPWM
	TCCR0A |= (1<<WGM00 | 1<<WGM01);
	TCCR2A |= (1<<WGM20 | 1<<WGM21);

	// Set clock source (prescaler)
	TCCR0B |= (1<<CS01);
	TCCR2B |= (1<<CS21);

	TCCR1A |= (1<<WGM10);
	TCCR1B |= (1<<CS10) | (1<<WGM12);
	TCCR1A |= (1<<WGM10);
	TCCR1B |= (1<<CS10) | (1<<WGM12);
	
	/* Ok, Google. end*/
}

void PWM_set(unsigned char channel, unsigned char data)
{
	if (!data) {
		PWM_disable(channel);
		return;
	} else {
		PWM_enable(channel);
	}
	switch (channel)
	{
		case 0:
			OCR2A = data;
			break;
		case 1:
			OCR1B = data;
			break;
		case 2:
			OCR1A = data;
			break;
		case 3:
			OCR0B = data;
			break;
		case 4:
			OCR0A = data;
			break;
		case 5:
			OCR2B = data;
			break;
	}
}


void PWM_enable(unsigned char channel)
{
	switch (channel)
	{
		case 0:
			TCCR2A |= 1<<COM2A1;
			break;
		case 1:
			TCCR1A |= 1<<COM1B1;
			break;
		case 2:
			TCCR1A |= 1<<COM1A1;
			break;
		case 3:
			TCCR0A |= 1<<COM0B1;
			break;
		case 4:
			TCCR0A |= 1<<COM0A1;
			break;
		case 5:
			TCCR2A |= 1<<COM2B1;
			break;
	}
}


void PWM_disable(unsigned char channel)
{
	switch (channel)
	{
		case 0:
			TCCR2A &= ~(1<<COM2A1);
			break;
		case 1:
			TCCR1A &= ~(1<<COM1B1);
			break;
		case 2:
			TCCR1A &= ~(1<<COM1A1);
			break;
		case 3:
			TCCR0A &= ~(1<<COM0B1);
			break;
		case 4:
			TCCR0A &= ~(1<<COM0A1);
			break;
		case 5:
			TCCR2A &= ~(1<<COM2B1);
			break;
	}
}